package com.grail.common.core.domain;

import com.grail.common.core.constant.ConstantsTwo;
import lombok.Data;

import java.io.Serializable;

/**
 * @ClassName Result
 * @Description 响应信息主体
 * @Author ChenLong
 * @Date 2023/2/23 16:08
 * @Version 1.0
 */
@Data
public class Result<T> implements Serializable {
    private static final long serialVersionUID = 1L;
    /** 成功 */
    public static final int SUCCESS = ConstantsTwo.SUCCESS;
    /** 失败 */
    public static final int FAIL = ConstantsTwo.ERROR;
    private int code;
    private String msg;
    private T data;
    public static <T> Result<T> success() {
        return restResult(null, SUCCESS, ConstantsTwo.SUCCESS_MSG);
    }
    public static <T> Result<T> success(T data) {
        return restResult(data, SUCCESS, ConstantsTwo.SUCCESS_MSG);
    }
    public static <T> Result<T> success(T data, String msg) {
        return restResult(data, SUCCESS, msg);
    }
    public static <T> Result<T> error() {
        return restResult(null, FAIL, ConstantsTwo.ERROR_MSG);
    }
    public static <T> Result<T> error(String msg) {
        return restResult(null, FAIL, msg);
    }
    public static <T> Result<T> error(T data) {
        return restResult(data, FAIL, ConstantsTwo.ERROR_MSG);
    }
    public static <T> Result<T> error(T data, String msg) {
        return restResult(data, FAIL, msg);
    }
    public static <T> Result<T> error(int code, String msg) {
        return restResult(null, code, msg);
    }
    private static <T> Result<T> restResult(T data, int code, String msg) {
        Result<T> apiResult = new Result<>();
        apiResult.setCode(code);
        apiResult.setData(data);
        apiResult.setMsg(msg);
        return apiResult;
    }
}
